import secrets
from unittest import mock

import pytest
from google.auth.credentials import Credentials


class MockCredentials(Credentials):
    "Mock credentials object used in place of real Google credentials"

    def __init__(self, token):
        self._post_refresh_token = token

    def refresh(self, request):
        self.token = self._post_refresh_token


@pytest.fixture
def mock_google_default_token():
    return secrets.token_urlsafe()


@pytest.fixture
def mock_google_impersonated_token():
    return secrets.token_urlsafe()


@pytest.fixture
def mock_google_id_token():
    return secrets.token_urlsafe()


@pytest.fixture
def mock_google_user_info(faker):
    return {"id": faker.slug(), "email": faker.email()}


@pytest.fixture
def mock_google_user_info_endpoint(
    mock_google_user_info, mock_google_default_token, requests_mock
):
    "Mock the https://www.googleapis.com/oauth2/v1/userinfo endpoint for the default user."
    requests_mock.get(
        f"https://www.googleapis.com/oauth2/v1/userinfo?access_token={mock_google_default_token}",
        headers={"Content-Type": "application/json"},
        json=mock_google_user_info,
    )


@pytest.fixture(autouse=True)
def mock_google_auth_default(monkeypatch, faker, mock_google_default_token):
    "Mock Google default credential provider."
    magic_mock = mock.MagicMock()
    magic_mock.return_value = (MockCredentials(mock_google_default_token), faker.slug())
    monkeypatch.setattr("google.auth.default", magic_mock)
    return magic_mock


@pytest.fixture(autouse=True)
def mock_google_auth_impersonated_credentials(monkeypatch, mock_google_impersonated_token):
    "Mock Google impersonated credential provider."
    magic_mock = mock.MagicMock()
    magic_mock.return_value = MockCredentials(mock_google_impersonated_token)
    monkeypatch.setattr("google.auth.impersonated_credentials.Credentials", magic_mock)
    return magic_mock


@pytest.fixture(autouse=True)
def mock_google_auth_id_token_credentials(monkeypatch, mock_google_id_token):
    "Mock Google impersonated credential provider."
    magic_mock = mock.MagicMock()
    magic_mock.return_value = MockCredentials(mock_google_id_token)
    monkeypatch.setattr("google.auth.impersonated_credentials.IDTokenCredentials", magic_mock)
    return magic_mock
